#include "TArtCalibNEBULA.hh" 
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtNEBULAParameters.hh"
#include "TArtRawEventObject.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"
#include "TArtMapper.h"

#include <TClonesArray.h>
#include <iostream>

//__________________________________________________________
TArtCalibNEBULA::TArtCalibNEBULA() : fReconstructed(false), fNEBULAParameters(0)
{
  TArtCore::Info(__FILE__,"Creating the NEBULA detector objects...");

  fStoreManager = TArtStoreManager::Instance();
  fNEBULAPlaArray = new TArtNEBULAPlaArray("TArtNEBULAPla", 200);
  fNEBULAPlaArray->SetOwner();
  fNEBULAPlaArray->SetName("NEBULAPla");
  fStoreManager->AddDataContainer(fNEBULAPlaArray);
  fNEBULAPlaParaArray = new TArtNEBULAPlaParaArray;
}

//__________________________________________________________
TArtCalibNEBULA::~TArtCalibNEBULA()
{
  delete fNEBULAPlaParaArray;
  delete fNEBULAPlaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::BeginEvent()
{
  if(!fNEBULAParameters){
    fNEBULAParameters = (TArtNEBULAParameters*)fStoreManager->FindParameters("NEBULAParameters");
  }
  LoadRawData();
}

//__________________________________________________________
void TArtCalibNEBULA::ReconstructData()
{
  for(int i=0;i<GetNumNEBULAPla();i++){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    const TArtNEBULAPlaPara* para = GetNEBULAPlaPara(i);

    unsigned int adc = pla->GetRawADC();
    unsigned int tdc = pla->GetRawTDC();

    if(tdc>0 && adc>0 && tdc<=4096 && adc<=4096){
      pla->SetDataState(1);
    }else{
      pla->SetDataState(-1);      
      continue;
    }

    Double_t fTime = tdc * para->GetTCal();
    Double_t fEnergy = (adc - para->GetQPed()) * para->GetQCal();

    pla->SetEnergy(fEnergy);
    pla->SetTime(fTime);
    
    pla->SetID(para->GetID());
    pla->SetUDLR(para->GetUDLR());
    pla->SetFpl(para->GetFpl());
    pla->SetDetectorName(para->GetDetectorName());
    pla->SetLayer(para->GetLayer());
    pla->SetSubLayer(para->GetSubLayer());
  }
  
  fReconstructed = true;
  return;
}

//__________________________________________________________
void TArtCalibNEBULA::EndEvent()
{
  ClearData();
}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPla() const
{
  return fNEBULAPlaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::GetNEBULAPla(int i) const
{
  return (TArtNEBULAPla*)fNEBULAPlaArray->At(i);
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::FindNEBULAPla(Int_t serialnumber) const
{
  for(int i=0; i<GetNumNEBULAPla(); i++){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    if(serialnumber == pla->GetSerialNumber()) return pla;
  }
  return NULL;
}

//__________________________________________________________
TArtNEBULAPlaArray* TArtCalibNEBULA::GetNEBULAPlaArray() const
{
  return fNEBULAPlaArray;
}

//__________________________________________________________
const TArtNEBULAPlaPara* TArtCalibNEBULA::GetNEBULAPlaPara(int i) const
{
  return (const TArtNEBULAPlaPara*)(*fNEBULAPlaParaArray)[i];
}

//__________________________________________________________
void TArtCalibNEBULA::ClearData()
{
  fNEBULAPlaArray->Clear();
  fNEBULAPlaParaArray->clear();
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibNEBULA::LoadRawData()
{
  TArtRawEventObject* fEvent = (TArtRawEventObject*)fStoreManager->FindDataContainer("RawEvent");
  ClearData();
  for(int i=0; i<fEvent->GetNumSeg(); ++i){
    TArtRawSegmentObject* seg = fEvent->GetSegment(i);
    int device = seg->GetDevice();
    if(SAMURAI == device){
      int device = seg->GetDevice();
      int fpl = seg->GetFP();
      int detector = seg->GetDetector();
      for(int j=0; j<seg->GetNumData(); ++j){
	TArtRawDataObject* d = seg->GetData(j);
	int geo = d->GetGeo(); 
	int ch = d->GetCh(); 
	unsigned int val = d->GetVal();
	TArtRIDFMap mm(fpl,detector,geo,ch);
	const TArtNEBULAPlaPara* para = fNEBULAParameters->FindNEBULAPlaPara(mm);
	if(!para){
	  TArtCore::Info(__FILE__,"Could not find TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}
	if(para->GetIgnore()){
	  TArtCore::Debug(__FILE__,"Ignore TArtNEBULAPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}
	int serialnumber = para->GetSerialNumber();
	TArtNEBULAPla* pla = FindNEBULAPla(serialnumber);
	if(!pla){
	  int npla = fNEBULAPlaArray->GetEntries();
	  pla = new ((*fNEBULAPlaArray)[npla]) TArtNEBULAPla();
	  pla->SetSerialNumber(serialnumber);
	  fNEBULAPlaParaArray->push_back(para);
	}

	// if(NEBULAA == detector)
	if(42 == detector){
	  pla->SetRawADC(val);
	}
	// if(NEBULAT == detector)
	if(41 == detector){
	  pla->SetRawTDC(val);
	}
      }
    }
  }
  return;
}

